OVERRIDE_FLAG	EQU	103Bh
NL		EQU	0009h
MTON		EQU	069Fh
EXIT		EQU	055Ah
OKMSG		EQU	0942h

; LPT
;	Bit0 [R] : RDA
;	Bit1 [R] : STA
;	Bit6 [W] : IRT
;	Bit7 [W] : RDP
;
LPT_DATA	EQU	0FFh
LPT_STATUS	EQU	0FEh

	ORG	0155h

;------------------------------------------------------------------
;	Entry of 'P' and 'T' command
;
;	IN:  A = Command (ASCII)
;------------------------------------------------------------------
cmd_p_t:
	LD	(OVERRIDE_FLAG), A	; 'T' or 'P'
	JP	0FA1h			; NL:print'OK!':JP 00ADh

;------------------------------------------------------------------
;	Patch entry: Called from RDI/RDD/WRI/WRD instead of MTON
;
;	IN:  HL = Buffer Address
;	     BC = Size
;	     D  = 0D2h(Read) / 0D7h(Write)
;------------------------------------------------------------------
entry:
	LD	A, (OVERRIDE_FLAG)
	CP	'P'
	JP	NZ, MTON
	LD	A, 0F8h		; Row for SHIFT+BREAK
	LD	(0E000h), A
	LD	E, 81h		; Break check & LPT status mask & loop count
	LD	A, D
	CP	0D2h		; Read?
	JR	Z, read_lpt

;------------------------------------------------------------------
;	Write to LPT
;
;	IN:  HL = Buffer Address
;	     BC = Size
;------------------------------------------------------------------
L1:
@@:	CALL	checkRDA	; wait until RDA==L
	JR	NC, @b

	LD	A, (HL)		; read from buffer
	INC	HL
	OUT	(LPT_DATA), A	; data out
	LD	A, E
	OUT	(LPT_STATUS), A	; set RDP=H

@@:	CALL	checkRDA	; wait until RDA==H
	JR	C, @b
	XOR	A
	OUT	(LPT_STATUS), A	; set RDP=L

	DEC	BC
	LD	A, B
	OR	C
	JR	NZ, L1		; A=0, Cy=0

xit:	POP	HL		; dispose original return address
	POP	HL
	POP	BC
	POP	DE
	JP	EXIT

checkRDA:			; OUT: Cy = RDA, A = [RDA]|000000|[STA]
	LD	A, (0E001h)
	AND	E
	IN	A, (LPT_STATUS)	; (Z flag is unchanged)
	RRCA			; (Z flag is unchanged)
	RET	NZ
	POP	HL		; dispose return address
	LD	A, E
	ADD	A, A		; (== LD A,2:SCF)
	OUT	(LPT_STATUS), A	; set RDP=L, IRT=L
	JR	xit

;------------------------------------------------------------------
;	Read from LPT
;
;	IN:  HL = Buffer Address
;	     BC = Size
;------------------------------------------------------------------
read_lpt:
	LD	A, 40h		; set IRT=H
	OUT	(LPT_STATUS), A	; request PC to send data
L3:
	LD	D, E		; (bit7 is ignored; bit0 is loop-end flag)
L2:
@@:	CALL	checkRDA
	JR	C, @b		; wait until RDA==L
	RRA			; get STA
	LD	A, E
	OUT	(LPT_STATUS), A	; set RDP=H
	RL	D
@@:	CALL	checkRDA
	JR	NC, @b		; wait until RDA==H
	AND	01h
	OUT	(LPT_STATUS), A	; set RDP=L
	RRA			; get STA
	RL	D
	JR	NC, L2

	LD	(HL), D		; store received data
	INC	HL
	DEC	BC
	LD	A, B
	OR	C
	JR	NZ, L3		; A=0, Cy=0
	JR	xit

	END

;L0085:	LD	HL, 038Dh	-->	LD	HL, 03D5h
;	LD	A, 0C3h		-->	LD	DE, 1038h
;	LD      (1038h),A	-->	LD	BC, 4
;	LD      (1039h),HL	-->	LDIR
;L00C9:	CP	'F'		-->	CP	'T'
;	JR	Z, L00F6	-->	JR	Z, L00D7
;L03D5:	DB	1,9,9,9,0Dh	-->	DB	0C3h, 8Dh, 03h, 'T'
;L0447:	CALL    MTON		-->	CALL	entry
;L04E6:	CALL    MTON		-->	CALL	entry
